package com.jpattern.orm.session;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.SQLException;

import javax.sql.DataSource;

import com.jpattern.logger.ILogger;
import com.jpattern.orm.dialect.DBType;
import com.jpattern.orm.dialect.DetermineDBType;
import com.jpattern.orm.logger.OrmLogger;

/**
 * 
 * @author Francesco Cina
 *
 * 21/mag/2011
 */
public abstract class SessionProvider {

	private final ILogger logger = OrmLogger.getOrmLogger(this.getClass());
	private DBType dbType;

	public abstract SessionStrategy getSessionStrategy();
	public abstract DBType getDBType();

	protected final DBType getDBType(final DataSource datasource) {
		if (this.dbType==null) {
			Connection connection = null;
			this.dbType = DBType.UNDEFINED;
			try
			{
				connection = datasource.getConnection();
				DatabaseMetaData metaData = connection.getMetaData();

				String driverName = metaData.getDriverName();
				String url = metaData.getURL();
				String databaseProductName = metaData.getDatabaseProductName();

				getLogger().info("getDbType", "driver name: " + driverName);
				getLogger().info("getDbType", "DB url: " + url);
				getLogger().info("getDbType", "DB product name: " + databaseProductName);

				this.dbType = new DetermineDBType().determineDBType(driverName, url, databaseProductName);
			}
			catch (SQLException ex)
			{
				getLogger().warn("getDbType", "Error while determining the database type");
			}
			finally
			{
				if (connection != null)
				{
					try
					{
						connection.close();
					}
					catch (SQLException ex)
					{
						// we ignore this one
					}
				}
			}
			getLogger().info("getDbType", "DB type is " + this.dbType);
		}
		return this.dbType;
	}

	public ILogger getLogger() {
		return this.logger;
	}
}
