package com.jpattern.orm.session.datasource;

import javax.sql.DataSource;

import com.jpattern.orm.dialect.DBType;
import com.jpattern.orm.session.SessionProvider;
import com.jpattern.orm.session.SessionStrategy;

/**
 * 
 * @author Francesco Cina
 *
 * 21/mag/2011
 */
public class DataSourceSessionProvider extends SessionProvider {

	private final ThreadLocal<IConnection> threadLocalConnection = new ThreadLocal<IConnection>();
	private final DataSource dataSource;

	public DataSourceSessionProvider(final DataSource dataSource) {
		this.dataSource = dataSource;
	}

	@Override
	public SessionStrategy getSessionStrategy() {
		return new DataSourceSessionStrategy(this);
	}

	final IConnection getConnection(final boolean readOnly, final IConnectionCaller connectionCaller) {
		getLogger().debug("getConnection", "new connection asked by " + connectionCaller );
		IConnection conn = this.threadLocalConnection.get();
		if ((conn==null) || !conn.isValid()) {
			getLogger().info("getConnection", "no valid connections found, a new one will be created");
			conn = new DataSourceConnection(this.dataSource, readOnly);
			this.threadLocalConnection.set(conn);
		}
		conn.addCaller(connectionCaller);
		return conn;
	}

	@Override
	public DBType getDBType() {
		return getDBType(this.dataSource);
	}

}
